import 'dart:async';

import 'package:flutter/material.dart';

import 'package:intl/date_symbol_data_local.dart';
import 'package:intl/intl.dart' show DateFormat;
import 'package:senior_design_tokens/senior_design_tokens.dart';

import './models/senior_event_calendar.dart';
import './senior_calendar_header.dart';
import './senior_weekday_row.dart';
import '../../components/senior_calendar/senior_calendar.dart';
import '../../themes/senior_theme.dart';
import '../../themes/senior_theme_provider.dart';

typedef MarkedDateIconBuilder<T> = Widget? Function(T event);
typedef void OnDayLongPressed(DateTime day);

/// This builder is called for every day in the calendar.
/// If you want to build only few custom day containers, return null for the days you want to leave with default looks
/// All characteristics like circle border are also applied to the custom day container [DayBuilder] provides.
/// (if supplied function returns null, Calendar's function will be called for [day]).
/// [isSelectable] - is between [SeniorCalendar.minSelectedDate] and [SeniorCalendar.maxSelectedDate]
/// [index] - DOES NOT equal day number! Index of the day built in current visible field
/// [isSelectedDay] - if the day is selected
/// [isToday] - if the day is similar to [DateTime.now()]
/// [isPrevMonthDay] - if the day is from previous month
/// [textStyle] - text style that would have been applied by the calendar if it was to build the day.
/// Example: if the user provided [SeniorCalendar.todayTextStyle] and [isToday] is true,
///   [SeniorCalendar.todayTextStyle] would be sent into [DayBuilder]'s [textStyle]. If user didn't
///   provide it, default [SeniorCalendar]'s textStyle would be sent. Same applies to all text styles like
///   [SeniorCalendar.prevDaysTextStyle], [SeniorCalendar.daysTextStyle] etc.
/// [isNextMonthDay] - if the day is from next month
/// [isThisMonthDay] - if the day is from next month
/// [day] - day being built.
/// [isWeekendDay] - if the day is a weekend day.
typedef Widget? DayBuilder(
    bool isSelectable,
    int index,
    bool isSelectedDay,
    bool isToday,
    bool isPrevMonthDay,
    TextStyle textStyle,
    bool isNextMonthDay,
    bool isThisMonthDay,
    bool isWeekendDay,
    DateTime day);

/// This builder is called for every weekday container (7 times, from Mon to Sun).
/// [weekday] - weekday built, from 0 to 6.
/// [weekdayName] - string representation of the weekday (Mon, Tue, Wed, etc).
typedef Widget WeekdayBuilder(int weekday, String weekdayName);

class SeniorCalendar<T extends EventCalendarInterface> extends StatefulWidget {
  final double viewportFraction;

  final double dayPadding;
  final double height;
  final double width;
  final DateTime? selectedDateTime;
  final DateTime? targetDateTime;
  final DateTime? minSelectedDate;
  final DateTime? maxSelectedDate;
  final Function? onHeaderTitlePressed;
  final Function? onLeftArrowPressed;
  final Function? onRightArrowPressed;
  final bool daysHaveCircularBorder;
  final bool disableDayPressed;
  final bool weekFormat;
  final bool showWeekDays;
  final bool showHeader;
  final bool showHeaderButton;
  final bool disableYesterdayPressed;
  final bool disableWeekendPressed;
  final bool disablePrevDayMonthPressed;
  final bool disableNextDayMonthPressed;
  final bool inactiveWeekendBeforeToday;
  final bool inactiveWeekendAfterToday;
  final bool headerTitleTouchable;
  final bool staticSixWeekFormat;
  final bool isScrollable;
  final bool shouldShowTransform;
  final bool weekendFormat;
  final bool showOnlyCurrentMonthDate;
  final bool pageSnapping;
  final bool showIconBehindDayText;
  final bool inactiveYesterdays;
  final bool highlightToday;
  final Function(DateTime)? onDayPressed;
  final String? headerText;
  final EdgeInsets headerMargin;
  final double childAspectRatio;
  final EdgeInsets weekDayMargin;
  final EdgeInsets weekDayPadding;
  final WeekdayBuilder? customWeekDayBuilder;
  final ScrollPhysics? customGridViewPhysics;
  final Function(DateTime)? onCalendarChanged;
  final String locale;
  final int? firstDayOfWeek;
  final WeekdayFormat weekDayFormat;
  final Axis scrollDirection;
  final OnDayLongPressed? onDayLongPressed;
  final CrossAxisAlignment dayCrossAxisAlignment;
  final MainAxisAlignment dayMainAxisAlignment;
  final ScrollPhysics pageScrollPhysics;
  final String? topTextCalendar;
  final String? bottomTextCalendar;
  final SeniorCalendarStyle? style;

  SeniorCalendar({
    Key? key,
    this.viewportFraction = 1.0,
    this.dayPadding = 1.0,
    this.height = 426.0,
    this.width = double.infinity,
    this.selectedDateTime,
    this.targetDateTime,
    this.daysHaveCircularBorder = true,
    this.onDayPressed,
    this.headerText,
    this.headerMargin = const EdgeInsets.symmetric(vertical: 10.0),
    this.childAspectRatio = 1.0,
    this.weekDayMargin = const EdgeInsets.only(bottom: 10.0),
    this.weekDayPadding = const EdgeInsets.all(6.0),
    this.customWeekDayBuilder,
    this.showWeekDays = true,
    this.weekFormat = false,
    this.showHeader = true,
    this.showHeaderButton = true,
    this.customGridViewPhysics,
    this.onCalendarChanged,
    this.locale = 'pt_BR',
    this.firstDayOfWeek = 0,
    this.minSelectedDate,
    this.maxSelectedDate,
    this.headerTitleTouchable = false,
    this.onHeaderTitlePressed,
    this.onLeftArrowPressed,
    this.onRightArrowPressed,
    this.weekDayFormat = WeekdayFormat.short,
    this.staticSixWeekFormat = false,
    this.isScrollable = true,
    this.scrollDirection = Axis.horizontal,
    this.showOnlyCurrentMonthDate = false,
    this.pageSnapping = false,
    this.onDayLongPressed,
    this.dayCrossAxisAlignment = CrossAxisAlignment.center,
    this.dayMainAxisAlignment = MainAxisAlignment.center,
    this.showIconBehindDayText = false,
    this.pageScrollPhysics = const BouncingScrollPhysics(),
    this.shouldShowTransform = true,
    this.weekendFormat = false,
    this.inactiveWeekendBeforeToday = true,
    this.inactiveWeekendAfterToday = true,
    this.topTextCalendar,
    this.bottomTextCalendar,
    this.disableWeekendPressed = false,
    this.disableDayPressed = false,
    this.disableYesterdayPressed = false,
    this.disableNextDayMonthPressed = false,
    this.disablePrevDayMonthPressed = false,
    this.inactiveYesterdays = true,
    this.highlightToday = true,
    this.style,
    TextStyle? selectedDayTextStyle,
  }) : super(key: key);

  @override
  _SeniorCalendarState<T> createState() => _SeniorCalendarState<T>();

  void pickCalendar(BuildContext context) {
    showModalBottomSheet<void>(
        backgroundColor: SeniorColors.pureWhite,
        isScrollControlled: true,
        elevation: 1,
        shape: const RoundedRectangleBorder(
          borderRadius:
              BorderRadius.vertical(top: Radius.circular(SeniorSpacing.normal)),
        ),
        context: context,
        builder: (BuildContext context) {
          return SeniorCalendar(
            topTextCalendar: topTextCalendar,
            bottomTextCalendar: bottomTextCalendar,
            onCalendarChanged: onCalendarChanged,
            locale: locale,
            firstDayOfWeek: firstDayOfWeek,
            minSelectedDate: minSelectedDate,
            maxSelectedDate: maxSelectedDate,
            headerTitleTouchable: headerTitleTouchable,
            onHeaderTitlePressed: onHeaderTitlePressed,
            onLeftArrowPressed: onLeftArrowPressed,
            onRightArrowPressed: onRightArrowPressed,
            weekDayFormat: weekDayFormat,
            staticSixWeekFormat: staticSixWeekFormat,
            isScrollable: isScrollable,
            scrollDirection: scrollDirection,
            showOnlyCurrentMonthDate: showOnlyCurrentMonthDate,
            pageSnapping: pageSnapping,
            onDayLongPressed: onDayLongPressed,
            dayCrossAxisAlignment: dayCrossAxisAlignment,
            dayMainAxisAlignment: dayMainAxisAlignment,
            showIconBehindDayText: showIconBehindDayText,
            pageScrollPhysics: pageScrollPhysics,
            shouldShowTransform: shouldShowTransform,
            weekendFormat: weekendFormat,
            viewportFraction: viewportFraction,
            dayPadding: dayPadding,
            height: height,
            width: width,
            selectedDateTime: selectedDateTime,
            targetDateTime: targetDateTime,
            daysHaveCircularBorder: daysHaveCircularBorder,
            disableDayPressed: disableDayPressed,
            disableYesterdayPressed: disableYesterdayPressed,
            onDayPressed: onDayPressed,
            headerText: headerText,
            headerMargin: headerMargin,
            childAspectRatio: childAspectRatio,
            weekDayMargin: weekDayMargin,
            weekDayPadding: weekDayPadding,
            customWeekDayBuilder: customWeekDayBuilder,
            showWeekDays: showWeekDays,
            weekFormat: weekFormat,
            showHeader: showHeader,
            showHeaderButton: showHeaderButton,
            customGridViewPhysics: customGridViewPhysics,
            disableNextDayMonthPressed: disableNextDayMonthPressed,
            disablePrevDayMonthPressed: disablePrevDayMonthPressed,
            disableWeekendPressed: disableWeekendPressed,
            inactiveWeekendAfterToday: inactiveWeekendAfterToday,
            inactiveWeekendBeforeToday: inactiveWeekendBeforeToday,
            inactiveYesterdays: inactiveYesterdays,
            highlightToday: highlightToday,
          );
        });
  }
}

enum WeekdayFormat {
  weekdays,
  standalone,
  short,
  standaloneShort,
  narrow,
  standaloneNarrow,
}

class _SeniorCalendarState<T extends EventCalendarInterface>
    extends State<SeniorCalendar<T>> with SingleTickerProviderStateMixin {
  late PageController _controller;
  late List<DateTime> _dates;
  late List<List<DateTime>> _weeks;
  DateTime _selectedDate = DateTime.now();
  late DateTime _targetDate;
  int _startWeekday = 0;
  int _endWeekday = 0;
  late DateFormat _localeDate;
  int _pageNum = 0;
  late DateTime minDate;
  late DateTime maxDate;
  late AnimationController animationController = AnimationController(
      vsync: this,
      duration: const Duration(milliseconds: 500),
      reverseDuration: const Duration(milliseconds: 500));

  /// When FIRSTDAYOFWEEK is 0 in dart-intl, it represents Monday. However it is the second day in the arrays of Weekdays.
  /// Therefore we need to add 1 modulo 7 to pick the right weekday from intl. (cf. [GlobalMaterialLocalizations])
  late int firstDayOfWeek;

  /// If the setState called from this class, don't reload the selectedDate, but it should reload selected date if called from external class

  @override
  void initState() {
    super.initState();
    initializeDateFormatting();

    minDate = widget.minSelectedDate ?? DateTime(2018);
    maxDate = widget.maxSelectedDate ??
        DateTime(
            DateTime.now().year + 1, DateTime.now().month, DateTime.now().day);

    final selectedDateTime = widget.selectedDateTime;
    if (selectedDateTime != null) {
      _selectedDate = selectedDateTime;
    }

    _init();

    _controller = PageController(
      initialPage: _pageNum,
      keepPage: true,
      viewportFraction: widget.viewportFraction,
    );

    _localeDate = DateFormat('MMMM yyyy', widget.locale);
    firstDayOfWeek = widget.firstDayOfWeek ??
        (_localeDate.dateSymbols.FIRSTDAYOFWEEK + 1) % 7;

    _setDate();
  }

  @override
  void didUpdateWidget(SeniorCalendar<T> oldWidget) {
    if (widget.targetDateTime != null && widget.targetDateTime != _targetDate) {
      _init();
      _setDate(_pageNum);
    }

    super.didUpdateWidget(oldWidget);
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  void _init() {
    final targetDateTime = widget.targetDateTime;
    if (targetDateTime != null) {
      if (targetDateTime.difference(minDate).inDays < 0) {
        _targetDate = minDate;
      } else if (targetDateTime.difference(maxDate).inDays > 0) {
        _targetDate = maxDate;
      } else {
        _targetDate = targetDateTime;
      }
    } else {
      _targetDate = _selectedDate;
    }
    if (widget.weekFormat) {
      _pageNum = _targetDate.difference(_firstDayOfWeek(minDate)).inDays ~/ 7;
    } else {
      _pageNum = (_targetDate.year - minDate.year) * 12 +
          _targetDate.month -
          minDate.month;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Padding(
      padding: const EdgeInsets.all(16.0),
      child: Container(
        //padding: const EdgeInsets.all(16.0),
        decoration: BoxDecoration(
          borderRadius: BorderRadius.circular(10.0),
          color: SeniorColors.pureWhite,
        ),
        //width: ,
        height: widget.height,
        child: Column(
          children: [
            Container(
              height: 4,
              width: 31,
              decoration: BoxDecoration(
                borderRadius: BorderRadius.circular(8.0),
                color: SeniorColors.secondaryColor500,
              ),
            ),
            const SizedBox(height: 16),
            widget.topTextCalendar != null
                ? Container(
                    width: double.infinity,
                    padding: const EdgeInsets.symmetric(horizontal: 16),
                    child: Text(
                      widget.topTextCalendar!,
                      style: SeniorTypography.titleCardRegular(
                          color: SeniorColors.neutralColor700),
                      textAlign: TextAlign.left,
                    ),
                  )
                : Container(),
            SeniorCalendarHeader(
              style: widget.style,
              showHeader: widget.showHeader,
              headerMargin: widget.headerMargin,
              headerTitle: buildHeaderText(),
              showHeaderButtons: widget.showHeaderButton,
              leftButtonActive: _pageNum > 0,
              rightButtonActive: !(_dates.length == (_pageNum + 1)),
              onLeftButtonPressed: () {
                widget.onLeftArrowPressed?.call();

                if (_pageNum > 0) {
                  _setDate(_pageNum - 1);
                }
              },
              onRightButtonPressed: () {
                widget.onRightArrowPressed?.call();

                if (widget.weekFormat) {
                  if (_weeks.length - 1 > _pageNum) {
                    _setDate(_pageNum + 1);
                  }
                } else {
                  if (_dates.length - 1 > _pageNum) {
                    _setDate(_pageNum + 1);
                  }
                }
              },
              onHeaderTitlePressed: widget.headerTitleTouchable
                  ? () {
                      final onHeaderTitlePressed = widget.onHeaderTitlePressed;
                      if (onHeaderTitlePressed != null) {
                        onHeaderTitlePressed();
                      } else {
                        _selectDateFromPicker();
                      }
                    }
                  : null,
            ),
            SeniorWeekdayRow(
              firstDayOfWeek,
              widget.customWeekDayBuilder,
              showWeekdays: widget.showWeekDays,
              weekdayFormat: widget.weekDayFormat,
              weekdayMargin: widget.weekDayMargin,
              weekdayPadding: widget.weekDayPadding,
              localeDate: _localeDate,
              style: widget.style,
            ),
            Expanded(
              child: PageView.builder(
                itemCount: widget.weekFormat ? _weeks.length : _dates.length,
                physics: widget.isScrollable
                    ? widget.pageScrollPhysics
                    : const NeverScrollableScrollPhysics(),
                onPageChanged: (index) {
                  _setDate(index);
                },
                controller: _controller,
                itemBuilder: (context, index) {
                  return widget.weekFormat
                      ? weekBuilder(index)
                      : builder(index);
                },
                pageSnapping: widget.pageSnapping,
              ),
            ),
            // SizedBox(height: 16),
            widget.bottomTextCalendar != null
                ? Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 16),
                    child: Container(
                      width: double.infinity,
                      child: Text(
                        widget.bottomTextCalendar!,
                        style: SeniorTypography.helper(
                            color: SeniorColors.neutralColor700),
                        textAlign: TextAlign.left,
                      ),
                    ),
                  )
                : Container()
          ],
        ),
      ),
    );
  }

  Widget getDefaultDayContainer({
    required bool isSelectable,
    required int index,
    required bool isSelectedDay,
    required bool isToday,
    required bool isPrevMonthDay,
    required bool isNextMonthDay,
    required bool isThisMonthDay,
    required DateTime now,
    required bool isYesterday,
    required bool isWeekendDay,
    required bool isTomorrow,
  }) {
    return Container(
      width: double.infinity,
      height: double.infinity,
      child: Row(
        crossAxisAlignment: widget.dayCrossAxisAlignment,
        mainAxisAlignment: widget.dayMainAxisAlignment,
        children: <Widget>[
          Text(
            '${now.day}',
            semanticsLabel: now.day.toString(),
            style: (isSelectedDay && !isToday)
                ? SeniorTypography.titleCardBold(
                    color: getDayColor(
                    isSelectable,
                    index,
                    isSelectedDay,
                    isToday,
                    isPrevMonthDay,
                    isNextMonthDay,
                    isThisMonthDay,
                    now,
                    isYesterday,
                    isWeekendDay,
                    isTomorrow,
                  ))
                : SeniorTypography.text(
                    color: getDayColor(
                      isSelectable,
                      index,
                      isSelectedDay,
                      isToday,
                      isPrevMonthDay,
                      isNextMonthDay,
                      isThisMonthDay,
                      now,
                      isYesterday,
                      isWeekendDay,
                      isTomorrow,
                    ),
                  ),
            maxLines: 1,
          ),
        ],
      ),
    );
  }

  Widget renderDay({
    required bool isSelectable,
    required int index,
    required bool isSelectedDay,
    required bool isToday,
    required bool isPrevMonthDay,
    required bool isNextMonthDay,
    required bool isThisMonthDay,
    required DateTime now,
    required bool isYesterday,
    required bool isWeekendDay,
    required bool isTomorrow,
  }) {
    return isYesterday
        ? Container(
            margin: const EdgeInsets.symmetric(vertical: 7),
            decoration: renderBoxDecoration(now),
            child: buildGestureDetector(
              isSelectable,
              index,
              isSelectedDay,
              isToday,
              isPrevMonthDay,
              isNextMonthDay,
              isThisMonthDay,
              now,
              isYesterday,
              isWeekendDay,
              isTomorrow,
            ),
          )
        : Container(
            margin: const EdgeInsets.all(7),
            child: buildGestureDetector(
              isSelectable,
              index,
              isSelectedDay,
              isToday,
              isPrevMonthDay,
              isNextMonthDay,
              isThisMonthDay,
              now,
              isYesterday,
              isWeekendDay,
              isTomorrow,
            ),
          );
  }

  AnimatedBuilder builder(int slideIndex) {
    _startWeekday = _dates[slideIndex].weekday - firstDayOfWeek;
    if (_startWeekday == 7) {
      _startWeekday = 0;
    }
    _endWeekday =
        DateTime(_dates[slideIndex].year, _dates[slideIndex].month + 1, 1)
                .weekday -
            firstDayOfWeek;
    final double screenWidth = MediaQuery.of(context).size.width;
    final int totalItemCount = widget.staticSixWeekFormat
        ? 42
        : DateTime(
              _dates[slideIndex].year,
              _dates[slideIndex].month + 1,
              0,
            ).day +
            _startWeekday +
            (7 - _endWeekday);

    final int year = _dates[slideIndex].year;
    final int month = _dates[slideIndex].month;

    return AnimatedBuilder(
      animation: _controller,
      builder: (context, child) {
        if (!widget.shouldShowTransform) {
          return child!;
        }
        double value = 1.0;
        if (_controller.position.haveDimensions) {
          value = _controller.page! - slideIndex;
          value = (1 - (value.abs() * .6)).clamp(0.75, 1.0);
        }

        return Center(
          child: SizedBox(
            height: Curves.easeOut.transform(value) * widget.height,
            width: Curves.easeOut.transform(value) * screenWidth,
            child: child,
          ),
        );
      },
      child: Stack(
        children: <Widget>[
          Container(
            width: double.infinity,
            height: double.infinity,
            child: GridView.count(
              physics: widget.customGridViewPhysics,
              crossAxisCount: 7,
              childAspectRatio: 1.0,
              children: List.generate(
                totalItemCount,
                (index) {
                  final selectedDateTime = _selectedDate;

                  final isToday =
                      DateTime.now().day == index + 1 - _startWeekday &&
                          DateTime.now().month == month &&
                          DateTime.now().year == year;

                  final isSelectedDay = selectedDateTime.year == year &&
                      selectedDateTime.month == month &&
                      selectedDateTime.day == index + 1 - _startWeekday;

                  final isPrevMonthDay = index < _startWeekday;
                  final isNextMonthDay = index >=
                      (DateTime(year, month + 1, 0).day) + _startWeekday;
                  final isThisMonthDay = !isPrevMonthDay && !isNextMonthDay;

                  DateTime now = DateTime(year, month, 1);

                  if (isPrevMonthDay && !widget.showOnlyCurrentMonthDate) {
                    now = now.subtract(Duration(days: _startWeekday - index));
                  } else if (isThisMonthDay) {
                    now = DateTime(year, month, index + 1 - _startWeekday);
                  } else if (!widget.showOnlyCurrentMonthDate) {
                    now = DateTime(year, month, index + 1 - _startWeekday);
                  } else {
                    return Container();
                  }
                  bool isSelectable = true;
                  if (now.millisecondsSinceEpoch <
                      minDate.millisecondsSinceEpoch)
                    isSelectable = false;
                  else if (now.millisecondsSinceEpoch >
                      maxDate.millisecondsSinceEpoch) {
                    isSelectable = false;
                  }
                  final bool isYesterday =
                      now.isBefore(DateTime.now()) && !isToday;
                  final bool isWeekendDay = now.weekday == DateTime.saturday ||
                      now.weekday == DateTime.sunday;
                  final bool isTomorrow =
                      now.isAfter(DateTime.now()) && !isToday;

                  return renderDay(
                    index: index,
                    isSelectedDay: isSelectedDay,
                    isToday: isToday,
                    isPrevMonthDay: isPrevMonthDay,
                    isNextMonthDay: isNextMonthDay,
                    isThisMonthDay: isThisMonthDay,
                    isSelectable: isSelectable,
                    now: now,
                    isYesterday: isYesterday,
                    isWeekendDay: isWeekendDay,
                    isTomorrow: isTomorrow,
                  );
                },
              ),
            ),
          ),
        ],
      ),
    );
  }

  AnimatedBuilder weekBuilder(int slideIndex) {
    final double screenWidth = MediaQuery.of(context).size.width;
    List<DateTime> weekDays = _weeks[slideIndex];

    weekDays = weekDays
        .map((weekDay) => weekDay.add(Duration(days: firstDayOfWeek)))
        .toList();

    return AnimatedBuilder(
        animation: _controller,
        builder: (context, child) {
          double value = 1.0;
          if (_controller.position.haveDimensions) {
            value = _controller.page! - slideIndex;
            value = (1 - (value.abs() * .5)).clamp(0.0, 1.0);
          }

          return Center(
            child: SizedBox(
              height: Curves.easeOut.transform(value) * widget.height,
              width: Curves.easeOut.transform(value) * screenWidth,
              child: child,
            ),
          );
        },
        child: Stack(
          children: <Widget>[
            Positioned(
              child: Container(
                width: double.infinity,
                height: double.infinity,
                child: GridView.count(
                  physics: widget.customGridViewPhysics,
                  crossAxisCount: 7,
                  childAspectRatio: widget.childAspectRatio,
                  padding: EdgeInsets.zero,
                  children: List.generate(weekDays.length, (index) {
                    /// last day of month + weekday
                    final bool isToday =
                        weekDays[index].day == DateTime.now().day &&
                            weekDays[index].month == DateTime.now().month &&
                            weekDays[index].year == DateTime.now().year;
                    final bool isSelectedDay =
                        _selectedDate.year == weekDays[index].year &&
                            _selectedDate.month == weekDays[index].month &&
                            _selectedDate.day == weekDays[index].day;
                    final bool isPrevMonthDay =
                        weekDays[index].month < _targetDate.month;
                    final bool isNextMonthDay =
                        weekDays[index].month > _targetDate.month;
                    final bool isThisMonthDay =
                        !isPrevMonthDay && !isNextMonthDay;
                    final bool isWeekendDay =
                        weekDays[index].weekday == DateTime.saturday ||
                            weekDays[index].weekday == DateTime.sunday;
                    final DateTime now = DateTime(weekDays[index].year,
                        weekDays[index].month, weekDays[index].day);

                    final bool isSelectable = !(now.millisecondsSinceEpoch <
                            minDate.millisecondsSinceEpoch ||
                        now.millisecondsSinceEpoch >
                            maxDate.millisecondsSinceEpoch);

                    final bool isYesterday =
                        now.isBefore(DateTime.now()) && !isToday;
                    final bool isTomorrow =
                        now.isAfter(DateTime.now()) && !isToday;
                    return renderDay(
                      index: index,
                      isSelectedDay: isSelectedDay,
                      isToday: isToday,
                      isPrevMonthDay: isPrevMonthDay,
                      isNextMonthDay: isNextMonthDay,
                      isThisMonthDay: isThisMonthDay,
                      isSelectable: isSelectable,
                      now: now,
                      isYesterday: isYesterday,
                      isWeekendDay: isWeekendDay,
                      isTomorrow: isTomorrow,
                    );
                  }),
                ),
              ),
            ),
          ],
        ));
  }

  List<DateTime> _getDaysInWeek([DateTime? selectedDate]) {
    selectedDate ??= new DateTime.now();

    final firstDayOfCurrentWeek = _firstDayOfWeek(selectedDate);
    final lastDayOfCurrentWeek = _lastDayOfWeek(selectedDate);

    return _daysInRange(firstDayOfCurrentWeek, lastDayOfCurrentWeek).toList();
  }

  DateTime _firstDayOfWeek(DateTime date) {
    final day = _createUTCMiddayDateTime(date);
    return day.subtract(new Duration(days: date.weekday % 7));
  }

  DateTime _lastDayOfWeek(DateTime date) {
    final day = _createUTCMiddayDateTime(date);
    return day.add(new Duration(days: 7 - day.weekday % 7));
  }

  DateTime _createUTCMiddayDateTime(DateTime date) {
    // Magic const: 12 is to maintain compatibility with date_utils
    return new DateTime.utc(date.year, date.month, date.day, 12, 0, 0);
  }

  Iterable<DateTime> _daysInRange(DateTime start, DateTime end) {
    var offset = start.timeZoneOffset;

    return List<int>.generate(end.difference(start).inDays, (i) => i + 1)
        .map((int i) {
      var d = start.add(Duration(days: i - 1));

      final timeZoneDiff = d.timeZoneOffset - offset;
      if (timeZoneDiff.inSeconds != 0) {
        offset = d.timeZoneOffset;
        d = d.subtract(new Duration(seconds: timeZoneDiff.inSeconds));
      }
      return d;
    });
  }

  void _onDayLongPressed(DateTime picked) {
    widget.onDayLongPressed?.call(picked);
  }

  void _onDayPressed(
      DateTime picked,
      bool isWeekendDay,
      bool isYesterday,
      bool isTomorrow,
      bool isPrevMonthDay,
      bool isNextMonthDay,
      bool isSelectedDay) {
    if (widget.disableDayPressed ||
        widget.disableYesterdayPressed && isYesterday ||
        widget.disableWeekendPressed && isWeekendDay ||
        widget.disableNextDayMonthPressed && isNextMonthDay ||
        widget.disablePrevDayMonthPressed && isPrevMonthDay) {
      return;
    }

    setState(() {
      _selectedDate = picked;
    });
    widget.onDayPressed?.call(picked);
  }

  Future _selectDateFromPicker() async {
    final DateTime? selected = await showDatePicker(
      context: context,
      initialDate: _selectedDate,
      firstDate: minDate,
      lastDate: maxDate,
    );

    if (selected != null) {
      // updating selected date range based on selected week
      setState(() {
        _selectedDate = selected;
      });
      widget.onDayPressed?.call(selected);
    }
  }

  void _setDatesAndWeeks() {
    /// Setup default calendar format
    final List<DateTime> date = [];
    int currentDateIndex = 0;
    for (int _cnt = 0;
        0 >=
            DateTime(minDate.year, minDate.month + _cnt)
                .difference(DateTime(maxDate.year, maxDate.month))
                .inDays;
        _cnt++) {
      date.add(DateTime(minDate.year, minDate.month + _cnt, 1));
      if (0 ==
          date.last
              .difference(DateTime(_targetDate.year, _targetDate.month))
              .inDays) {
        currentDateIndex = _cnt;
      }
    }

    /// Setup week-only format
    final List<List<DateTime>> week = [];
    for (int _cnt = 0;
        0 >=
            minDate
                .add(Duration(days: 7 * _cnt))
                .difference(maxDate.add(const Duration(days: 7)))
                .inDays;
        _cnt++) {
      week.add(_getDaysInWeek(minDate.add(new Duration(days: 7 * _cnt))));
    }

    _startWeekday = date[currentDateIndex].weekday - firstDayOfWeek;

    if (_startWeekday == 7) {
      _startWeekday = 0;
    }
    _endWeekday = DateTime(date[currentDateIndex].year,
                date[currentDateIndex].month + 1, 1)
            .weekday -
        firstDayOfWeek;
    _dates = date;
    _weeks = week;
  }

  void _setDate([int page = -1]) {
    if (page == -1) {
      setState(() {
        _setDatesAndWeeks();
      });
    } else {
      if (widget.weekFormat) {
        setState(() {
          _pageNum = page;
          _targetDate = _weeks[page].first;
        });

        _controller.animateToPage(page,
            duration: const Duration(milliseconds: 1),
            curve: const Threshold(0.0));
      } else {
        setState(() {
          _pageNum = page;
          _targetDate = _dates[page];
          _startWeekday = _dates[page].weekday - firstDayOfWeek;
          _endWeekday = _lastDayOfWeek(_dates[page]).weekday - firstDayOfWeek;
        });
        _controller.animateToPage(page,
            duration: const Duration(milliseconds: 1),
            curve: const Threshold(0.0));
      }

      //call callback
      final onCalendarChanged = widget.onCalendarChanged;
      if (onCalendarChanged != null) {
        WidgetsBinding.instance.addPostFrameCallback((_) {
          onCalendarChanged(
              !widget.weekFormat ? _dates[page] : _weeks[page][firstDayOfWeek]);
        });
      }
    }
  }

  Color getDayColor(
    bool isSelectable,
    int index,
    bool isSelectedDay,
    bool isToday,
    bool isPrevMonthDay,
    bool isNextMonthDay,
    bool isThisMonthDay,
    DateTime now,
    bool isYesterday,
    bool isWeekendDay,
    bool isTomorrow,
  ) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme =
        SeniorThemeProvider.getDefaultTheme(context);

    if (isSelectedDay) {
      if (isToday) {
        return widget.style?.todayColor ??
            theme?.calendarTheme?.style?.todayColor ??
            defaultTheme.calendarTheme?.style?.todayColor ??
            Colors.black;
      } else {
        return widget.style?.selectedDayTextColor ??
            theme?.calendarTheme?.style?.selectedDayTextColor ??
            defaultTheme.calendarTheme?.style?.selectedDayTextColor ??
            Colors.black;
      }
    } else {
      if ((_localeDate.dateSymbols.WEEKENDRANGE
              .contains((index - 1 + firstDayOfWeek) % 7)) &&
          !isSelectedDay &&
          isThisMonthDay &&
          !isToday) {
        if (isSelectable && !widget.weekendFormat) {
          return widget.style?.weekendColor ??
              theme?.calendarTheme?.style?.weekendColor ??
              defaultTheme.calendarTheme?.style?.weekendColor ??
              Colors.black;
        } else {
          return widget.style?.inactiveWeekendColor ??
              theme?.calendarTheme?.style?.inactiveWeekendColor ??
              defaultTheme.calendarTheme?.style?.inactiveWeekendColor ??
              Colors.black;
        }
      } else {
        if (!isSelectable && now.day != DateTime.now().day) {
          return widget.style?.inactiveDaysColor ??
              theme?.calendarTheme?.style?.inactiveDaysColor ??
              defaultTheme.calendarTheme?.style?.inactiveDaysColor ??
              Colors.black;
        } else {
          if (isPrevMonthDay) {
            return widget.style?.prevDaysMonthColor ??
                theme?.calendarTheme?.style?.prevDaysMonthColor ??
                defaultTheme.calendarTheme?.style?.prevDaysMonthColor ??
                Colors.black;
          } else {
            if (isNextMonthDay) {
              return widget.style?.nextDaysMonthColor ??
                  theme?.calendarTheme?.style?.nextDaysMonthColor ??
                  defaultTheme.calendarTheme?.style?.nextDaysMonthColor ??
                  Colors.black;
            } else {
              if (isToday) {
                return widget.style?.todayColor ??
                    theme?.calendarTheme?.style?.todayColor ??
                    defaultTheme.calendarTheme?.style?.todayColor ??
                    Colors.black;
              } else {
                return widget.style?.daysColor ??
                    theme?.calendarTheme?.style?.daysColor ??
                    defaultTheme.calendarTheme?.style?.daysColor ??
                    Colors.black;
              }
            }
          }
        }
      }
    }
  }

  Widget getDayContainer(
    bool isSelectable,
    int index,
    bool isSelectedDay,
    bool isToday,
    bool isPrevMonthDay,
    bool isNextMonthDay,
    bool isThisMonthDay,
    DateTime now,
    bool isYesterday,
    bool isWeekendDay,
    bool isTomorrow,
  ) {
    return getDefaultDayContainer(
      index: index,
      isSelectable: isSelectable,
      isSelectedDay: isSelectedDay,
      isToday: isToday,
      isPrevMonthDay: isPrevMonthDay,
      isNextMonthDay: isNextMonthDay,
      isThisMonthDay: isThisMonthDay,
      now: now,
      isYesterday: isYesterday,
      isWeekendDay: isWeekendDay,
      isTomorrow: isTomorrow,
    );
  }

  // Coloca a primeira letra da string em maiúscula
  String capitalize(String string) {
    return '${string[0].toUpperCase()}${string.substring(1).toLowerCase()}';
  }

  OutlinedBorder renderDayBorder(
    bool isSelectedDay,
    bool isToday,
    bool isPrevMonthDay,
    bool isNextMonthDay,
    bool isSelectable,
    DateTime now,
    bool isYesterday,
    bool isWeekendDay,
    bool isTomorrow,
  ) {
    if (widget.daysHaveCircularBorder) {
      return CircleBorder(
        side: BorderSide(
          color: renderDayBorderColor(
            isSelectedDay,
            isToday,
            isPrevMonthDay,
            isNextMonthDay,
            isSelectable,
            isYesterday,
            isWeekendDay,
            isTomorrow,
          ),
          width: 2,
        ),
      );
    } else {
      return RoundedRectangleBorder(
          side: BorderSide(
        color: renderDayBorderColor(
          isSelectedDay,
          isToday,
          isPrevMonthDay,
          isNextMonthDay,
          isSelectable,
          isYesterday,
          isWeekendDay,
          isTomorrow,
        ),
      ));
    }
  }

  Color renderDayBorderColor(
    bool isSelectedDay,
    bool isToday,
    bool isPrevMonthDay,
    bool isNextMonthDay,
    bool isSelectable,
    bool isYesterday,
    bool isWeekendDay,
    isTomorrow,
  ) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme =
        SeniorThemeProvider.getDefaultTheme(context);

    if (isSelectedDay) {
      if (isToday && widget.highlightToday) {
        return widget.style?.todayBorderColor ??
            theme?.calendarTheme?.style?.todayBorderColor ??
            defaultTheme.calendarTheme?.style?.todayBorderColor ??
            Colors.black;
      } else {
        return widget.style?.selectedDayBorderColor ??
            theme?.calendarTheme?.style?.selectedDayBorderColor ??
            defaultTheme.calendarTheme?.style?.selectedDayBorderColor ??
            Colors.black;
      }
    } else if (isToday && widget.highlightToday) {
      return widget.style?.todayBorderColor ??
          theme?.calendarTheme?.style?.todayBorderColor ??
          defaultTheme.calendarTheme?.style?.todayBorderColor ??
          Colors.black;
    } else if (isPrevMonthDay) {
      return widget.style?.prevMonthDayBorderColor ??
          theme?.calendarTheme?.style?.prevMonthDayBorderColor ??
          defaultTheme.calendarTheme?.style?.prevMonthDayBorderColor ??
          Colors.black;
    } else if (isNextMonthDay) {
      return widget.style?.nextMonthDayBorderColor ??
          theme?.calendarTheme?.style?.nextMonthDayBorderColor ??
          defaultTheme.calendarTheme?.style?.nextMonthDayBorderColor ??
          Colors.black;
    } else {
      return widget.style?.thisMonthDayBorderColor ??
          theme?.calendarTheme?.style?.thisMonthDayBorderColor ??
          defaultTheme.calendarTheme?.style?.thisMonthDayBorderColor ??
          Colors.black;
    }
  }

  Color renderBackgroundColor(
    bool isSelectedDay,
    bool isToday,
    bool isSelectable,
    DateTime now,
    bool isYesterday,
    bool isWeekendDay,
    isTomorrow,
  ) {
    final SeniorTheme? theme = SeniorThemeProvider.getTheme(context);
    final SeniorTheme defaultTheme =
        SeniorThemeProvider.getDefaultTheme(context);

    if (isSelectedDay) {
      if (isToday && widget.highlightToday) {
        return widget.style?.todayButtonColor ??
            theme?.calendarTheme?.style?.todayButtonColor ??
            defaultTheme.calendarTheme?.style?.todayButtonColor ??
            Colors.white;
      } else {
        return widget.style?.selectedDayButtonColor ??
            theme?.calendarTheme?.style?.selectedDayButtonColor ??
            defaultTheme.calendarTheme?.style?.selectedDayButtonColor ??
            Colors.white;
      }
    } else if (isToday && widget.highlightToday) {
      return widget.style?.todayButtonColor ??
          theme?.calendarTheme?.style?.todayButtonColor ??
          defaultTheme.calendarTheme?.style?.todayButtonColor ??
          Colors.white;
    } else if (isYesterday) {
      return Colors.transparent;
    } else {
      return widget.style?.daysButtonColor ??
          theme?.calendarTheme?.style?.daysButtonColor ??
          defaultTheme.calendarTheme?.style?.daysButtonColor ??
          Colors.white;
    }
  }

  String buildHeaderText() {
    if (widget.headerText != null) {
      return widget.headerText!;
    } else {
      if (widget.weekFormat) {
        return '${_localeDate.format(_weeks[_pageNum].first)}';
      } else {
        return capitalize(_localeDate.format(_dates[_pageNum]));
      }
    }
  }

  Widget buildGestureDetector(
    bool isSelectable,
    int index,
    bool isSelectedDay,
    bool isToday,
    bool isPrevMonthDay,
    bool isNextMonthDay,
    bool isThisMonthDay,
    DateTime now,
    bool isYesterday,
    bool isWeekendDay,
    bool isTomorrow,
  ) {
    return GestureDetector(
      onLongPress: () => _onDayLongPressed(now),
      child: TextButton(
        style: TextButton.styleFrom(
          shape: renderDayBorder(
            isSelectedDay,
            isToday,
            isPrevMonthDay,
            isNextMonthDay,
            isSelectable,
            now,
            isYesterday,
            isWeekendDay,
            isTomorrow,
          ),
          backgroundColor: renderBackgroundColor(
            isSelectedDay,
            isToday,
            isSelectable,
            now,
            isYesterday,
            isWeekendDay,
            isTomorrow,
          ),
          padding: EdgeInsets.all(widget.dayPadding),
        ),
        onPressed: () => _onDayPressed(now, isWeekendDay, isYesterday,
            isTomorrow, isPrevMonthDay, isNextMonthDay, isSelectedDay),
        child: Stack(
          children: widget.showIconBehindDayText
              ? <Widget>[
                  getDayContainer(
                    isSelectable,
                    index,
                    isSelectedDay,
                    isToday,
                    isPrevMonthDay,
                    isNextMonthDay,
                    isThisMonthDay,
                    now,
                    isYesterday,
                    isWeekendDay,
                    isTomorrow,
                  ),
                ]
              : <Widget>[
                  getDayContainer(
                    isSelectable,
                    index,
                    isSelectedDay,
                    isToday,
                    isPrevMonthDay,
                    isNextMonthDay,
                    isThisMonthDay,
                    now,
                    isYesterday,
                    isWeekendDay,
                    isTomorrow,
                  ),
                ],
        ),
      ),
    );
  }

  BoxDecoration renderBoxDecoration(DateTime now) {
    final DateTime dateNow = DateTime.now();

    final bool isToday = DateTime(now.year, now.month, now.day + 1) ==
        DateTime(dateNow.year, dateNow.month, dateNow.day);
    final bool fistWeekday = (now.weekday == widget.firstDayOfWeek) ||
        (now.weekday == 7 && widget.firstDayOfWeek == 0);
    bool lastWeekday = (now.weekday == widget.firstDayOfWeek! - 1) ||
        (now.weekday == 6 && widget.firstDayOfWeek == 0);

    if (now.weekday == 7 && widget.firstDayOfWeek! + 6 == 7) {
      lastWeekday = true;
    }

    BoxDecoration boxDecoration = const BoxDecoration();

    if (widget.inactiveYesterdays) {
      if (isToday) {
        boxDecoration = BoxDecoration(
            color: SeniorColors.neutralColor100,
            borderRadius: fistWeekday
                ? BorderRadius.circular(50)
                : const BorderRadius.horizontal(right: Radius.circular(50)));
      } else if ((fistWeekday || lastWeekday) && widget.inactiveYesterdays) {
        boxDecoration = BoxDecoration(
          color: SeniorColors.neutralColor100,
          borderRadius: fistWeekday
              ? const BorderRadius.horizontal(left: Radius.circular(50))
              : const BorderRadius.horizontal(right: Radius.circular(50)),
        );
      } else {
        boxDecoration = BoxDecoration(
          color: SeniorColors.neutralColor100,
          borderRadius: isToday
              ? const BorderRadius.horizontal(right: Radius.circular(50))
              : BorderRadius.zero,
        );
      }
    }
    return boxDecoration;
  }
}
